
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v5.014.
use strict;
use warnings;

use Module::Build 0.3601;
use lib qw{inc}; use My::ModuleBuild;

my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.4005"
  },
  "configure_requires" => {
    "ExtUtils::CChecker" => 0,
    "File::ShareDir" => 0,
    "Module::Build" => "0.3601"
  },
  "dist_abstract" => "Some useful pointer utilities when writing FFI modules",
  "dist_author" => [
    "Graham Ollis <plicease\@cpan.org>"
  ],
  "dist_name" => "FFI-Util",
  "dist_version" => "0.07",
  "license" => "perl",
  "module_name" => "FFI::Util",
  "recommends" => {},
  "recursive_test_files" => 1,
  "requires" => {
    "Exporter::Tidy" => 0,
    "FFI::Raw" => "0.28",
    "File::ShareDir" => 0,
    "Math::Int64" => 0,
    "Module::Build" => "0.4005",
    "base" => 0,
    "perl" => "v5.10.0"
  },
  "script_files" => [],
  "share_dir" => {
    "dist" => "share"
  },
  "test_requires" => {
    "Test::More" => 0
  }
);


my %fallback_build_requires = (
  "Module::Build" => "0.4005",
  "Test::More" => 0
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = My::ModuleBuild->new(%module_build_args);

$build->create_build_script;
