#ifndef FFI_UTIL_CONFIG_H
#define FFI_UTIL_CONFIG_H

#ifdef HAS_STDINT_H
#include <stdint.h>
#endif
#ifdef HAS_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAS_STDDEF_H
#include <stddef.h>
#endif
#ifdef HAS_STRING_H
#include <string.h>
#endif
#ifdef HAS_TIME_H
#include <time.h>
#endif
#ifdef HAS_SYS_STAT_H
#include <sys/stat.h>
#endif

#endif
